# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['myopy']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'myopy',
    'version': '0.1.4',
    'description': 'myopy, run blind python files.',
    'long_description': "[![tests](https://github.com/loiccoyle/myopy/workflows/tests/badge.svg)](https://github.com/loiccoyle/myopy/actions) [![pypi](https://img.shields.io/pypi/v/myopy)](https://pypi.org/project/myopy/)\n\n# myopy\n\n> Run blind python files.\n\nThis single class package, provides python objects to a python file at run time. This is ideal for configuration files where the user does not need to know where an object comes from or how to initialize it. It allows the python file to be blind to the origin of it's objects, removing the need for imports, object initializations or convoluted subclassing.\n\nThis is pretty much a standalone clone of the way the amazing [qutebrowser](https://github.com/qutebrowser/qutebrowser) handles it's config files.\n\nFeel free to copy paste the `PyFile` class if you don't want the added dependency.\n\n# Installation\n```\npip install myopy\n```\n\n# Usage\n\nSay you want to allow the user to change a `dict` containing some settings for an application in a configuration file called `config.py`:\n\nIn the application you would have something along the lines of:\n\n```python\nfrom myopy import PyFile\n\nuser_dict = {'something': 2}\n\nconfig = PyFile('path/to/config.py')\nconfig.provide('settings', user_dict)  # we provide the config file the user_dict in the settings variable\nmodule = config.run()  # returns a module object\nprint('after running config: ', user_dict)\nprint('module: ', module)\n```\nAnd in the user facing `config.py`, the `user_dict` object would be provided in the `settings` variable, and the user can change its values at will:\n```python\nprint('in config: ', settings)\nsettings['something_else'] = 4\nsettings['something'] = 3\n```\n\nThe output would be:\n```\nin config: {'something': 2}\nafter running config: {'something': 3, 'something_else': 4}\nmodule: <module 'config' from 'path/to/config.py'>\n```\nthe `user_dict` is modified in place.\n\n",
    'author': 'Loic Coyle',
    'author_email': 'loic.coyle@hotmail.fr',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/loiccoyle/myopy',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import sys
import typing
from contextlib import contextmanager


# from qutebrowser configfiles.py
@contextmanager
def saved_sys_properties() -> typing.Iterator[None]:
    """Save various sys properties such as sys.path and sys.modules."""
    old_path = sys.path.copy()
    old_modules = sys.modules.copy()

    try:
        yield
    finally:
        sys.path = old_path
        for module in set(sys.modules).difference(old_modules):
            del sys.modules[module]




import sys
import typing
import logging
from types import ModuleType, CodeType
from pathlib import Path

from .utils import saved_sys_properties


class PyFile:
    def __init__(self, file_path: Path, raising: bool=True, verbose: bool=True):
        """Python file handler. Makes python objects accessible to the python
        file, compiles and runs it.

        Parameters:
            file_path: python file.
            verbose: If True, will print any compilation/runtime exception.
            raising: If True, will raise any compilation/runtime exception.
        """
        self.file_path = Path(file_path)
        self.module = self._init_module()
        self._raising = raising
        self._verbose = verbose
        self._log = logging.getLogger(__name__)

    def _init_module(self):
        module = ModuleType(self.file_path.stem)
        module.__file__ = str(self.file_path)
        return module

    def provide(self, attr:str, obj: typing.Any):
        """Provide 'obj' to the python file as 'attr'.

        Parameters:
            attr: name of the attribute in which to provide 'obj' to the python
                file.
            obj: object to provide.
        """
        setattr(self.module, attr, obj)
        self._log.debug(f'Providing "{obj}" to "{self.file_path}" as "{attr}".')

    def _compile(self) -> CodeType:
        """Compile python file.
        """
        with self.file_path.open('r') as f:
            source_code = f.read()
        ast = compile(source_code, self.file_path, 'exec')
        return ast

    def _run(self, ast: CodeType):
        """Run python file.

        Parameters:
            ast: compiled python code.
        """
        with saved_sys_properties():
            config_dir = str(self.file_path.parent)
            if config_dir not in sys.path:
                sys.path.insert(0, str(self.file_path.parent))
            exec(ast, self.module.__dict__)

    def _maybe_print_maybe_raise(
            self,
            func: typing.Callable,
            prefix: str=""
            ) -> typing.Any:
        try:
            return func()
        except Exception as e:
            if self._verbose:
                self._log.error(prefix + str(e))
            if self._raising:
                raise e

    def run(self) -> dict:
        """Compile and run python file, with access to the provided objects.

        Returns:
            module: module of the python file.
        """
        ast = self._maybe_print_maybe_raise(self._compile,
                prefix=f"Error at compilation in \"{self.file_path}\": ")
        self._maybe_print_maybe_raise(lambda: self._run(ast),
                prefix=f"Error at execution in \"{self.file_path}\": ")
        return self.module

